; blinking effect number "23"
; 19 LEDs, driven by charlieplexing
; ATTiny11L @ 1 MHz (internal R/C osc)

;PORTS
;  PB0-4 = row/column 1-5 (low = column, high = row)

;MATRIX
; ,---,---,---,---,---,
; | D | 5 | 9 |13 |17 |
; }---+---+---+---+---{
; | 1 | D |10 |14 |18 |
; }---+---+---+---+---{   # = LED number
; | 2 | 6 | D |15 |19 |   D = Diode (for charlieplexing)
; }---+---+---+---+---{   - = no LED (unused)
; | 3 | 7 |11 | D | - |
; }---+---+---+---+---{
; | 4 | 8 |12 |16 | D |
; '---'---'---'---'---'

;LEDS
;
;   2 3     11 12 13
; 1     4       14
;       5     15
;     6         16
;   7            17
;  8 9 10   19 18

;ANIMATIONS:
;  3 bytes per frame (A..S = LED 1..19)
;  #1: 0bCBA1wwww   duration = (wwww + 1) * 50 ms
;  #2: 0bKJIHGFED
;  #3: 0bSRQPONML
;
;  1 byte per command (x = don't care)
;  #1: 0bccc0LLLL
;    ccc = 000  ->  no operation (ignored), LLLL = xxxx
;    ccc = xx1  ->  start loop, LLLL = number of loops (0 = play once, 15 = play 16 times)
;    ccc = x10  ->  end of loop, LLLL = xxxx
;    ccc = 100  ->  end of animations (restart with first animation), LLLL = xxxx

;VERSION HISTORY
;  v0.1  20061018  Arne Rossius

.def	temp = R16
.def	temp2 = R17
.def	mux1 = R18
.def	mux2 = R19
.def	mux3 = R20
.def	mux4 = R21
.def	mux5 = R22
.def	mux = R23
.def	time = R24
.def	loops = R25

.def	YL = R28
.def	YH = R29

.equ	NOOP = 0x00
.equ	LOOP = 0x20-1 ;add 1 for "0 loops" = "play once" ... 16 for "play 16x"
.equ	LOOPEND = 0x40
.equ	END = 0x80

.include "../include/tn11def.inc"

.cseg
.org 0x000
	rjmp	start

animations:
.include "drop_in.asm"
.db NOOP, LOOP+5
.include "blink.asm"
.db LOOPEND, LOOP+3
.include "shiftreg.asm"
.db LOOPEND, LOOP+7
.include "alternating.asm"
.db LOOPEND, LOOP+2
.include "dual_pendulum2.asm"
.db LOOPEND, LOOP+3
.include "fill_v.asm"
.db LOOPEND, LOOP+2
.include "worm.asm"
.db LOOPEND, LOOP+10
.include "halves.asm"
.db LOOPEND, LOOP+5
.include "dual_chaser.asm"
.db LOOPEND, LOOP+3
.include "fill_h.asm"
.db LOOPEND, LOOP+2
.include "pendulum.asm"
.db LOOPEND, LOOP+15
.include "scanlines.asm"
.db LOOPEND, NOOP
.include "fill_split.asm"
.db END, END

start:
	;Port setup
	ldi	temp, 0x3F
	out	DDRB, temp
	ldi	temp, 0x00
	out	PORTB, temp

main_start:
	;restart animation sequence
	ldi	loops, 0
	ldi	ZL, LOW(animations*2)
	ldi	ZH, HIGH(animations*2)

main_loop:
	;play animations
	lpm
	subi	ZL, -1
	brcs	PC+2
	inc	ZH
	sbrc	R0, 4
	rjmp	frame_init

	;not a frame but a command
	sbrc	R0, 5 ;loop
	rjmp	loop_init
	sbrc	R0, 6 ;loop-end
	rjmp	loop_end
	sbrc	R0, 7 ;end of animations (restart)
	rjmp	main_start
	rjmp	main_loop

loop_end:
	cpi	loops, 0
	breq	main_loop
	dec	loops
	mov	ZL, YL
	mov	ZH, YH
	rjmp	main_loop
	
loop_init:
	mov	loops, R0
	andi	loops, 0x0F
	mov	YH, ZH
	mov	YL, ZL
	rjmp	main_loop



frame_init:
	;new frame data
	clr	mux1
	clr	mux2
	clr	mux3
	clr	mux4
	clr	mux5
			;byte 1 = LED 1..3, duration
	mov	time, R0
	andi	time, 0x0F
	inc	time
	sbrc	R0, 5
	ori	mux1, 0x02
	sbrc	R0, 6
	ori	mux1, 0x04
	sbrc	R0, 7
	ori	mux1, 0x08
	lpm		;byte 2 = LED 4..11
	subi	ZL, -1
	brcs	PC+2
	inc	ZH
	sbrc	R0, 0
	ori	mux1, 0x10
	sbrc	R0, 1
	ori	mux2, 0x01
	sbrc	R0, 2
	ori	mux2, 0x04
	sbrc	R0, 3
	ori	mux2, 0x08
	sbrc	R0, 4
	ori	mux2, 0x10
	sbrc	R0, 5
	ori	mux3, 0x01
	sbrc	R0, 6
	ori	mux3, 0x02
	sbrc	R0, 7
	ori	mux3, 0x08
	lpm		;byte 3 = LED 12..19
	subi	ZL, -1
	brcs	PC+2
	inc	ZH
	sbrc	R0, 0
	ori	mux3, 0x10
	sbrc	R0, 1
	ori	mux4, 0x01
	sbrc	R0, 2
	ori	mux4, 0x02
	sbrc	R0, 3
	ori	mux4, 0x04
	sbrc	R0, 4
	ori	mux4, 0x10
	sbrc	R0, 5
	ori	mux5, 0x01
	sbrc	R0, 6
	ori	mux5, 0x02
	sbrc	R0, 7
	ori	mux5, 0x04

frame_loop:
	;display frame, $time x ~50ms [~120 Hz / 6]
	ldi	mux, 6
mux_loop:
	out	DDRB, mux1
	sbi	DDRB, 0
	out	PORTB, mux1
	rcall	muxwait
	ldi	temp, 0x00
	out	PORTB, temp
	
	out	DDRB, mux2
	sbi	DDRB, 1
	out	PORTB, mux2
	rcall	muxwait
	ldi	temp, 0x00
	out	PORTB, temp
	
	out	DDRB, mux3
	sbi	DDRB, 2
	out	PORTB, mux3
	rcall	muxwait
	ldi	temp, 0x00
	out	PORTB, temp
	
	out	DDRB, mux4
	sbi	DDRB, 3
	out	PORTB, mux4
	rcall	muxwait
	ldi	temp, 0x00
	out	PORTB, temp
	
	out	DDRB, mux5
	sbi	DDRB, 4
	out	PORTB, mux5
	rcall	muxwait
	ldi	temp, 0x00
	out	PORTB, temp

	dec	mux
	brne	mux_loop
	
	dec	time
	brne	frame_loop

	rjmp	main_loop

;--------------------

muxwait:
	clr	temp2
muxwait_loop:
	nop
	nop
	nop
	dec	temp2
	brne	muxwait_loop
	ret
